@@ -31,6 +31,7 @@ import logging
 from six.moves.configparser import ConfigParser
 
 import json
+import multiprocessing
 import netaddr
 import os
 import os.path
@@ -241,13 +242,14 @@ def playbook_verbosity(self):
 
 
 def run_ansible_playbook(playbook, inventory, workdir, playbook_dir=None,
-                         connection='smart', output_callback='yaml',
+                         connection='smart', output_callback='tripleo_dense',
                          ssh_user='root', key=None, module_path=None,
                          limit_hosts=None, tags=None, skip_tags=None,
                          verbosity=0, quiet=False, extra_vars=None,
                          extra_vars_file=None, plan='overcloud',
                          gathering_policy='smart', extra_env_variables=None,
-                         parallel_run=False, callback_whitelist=None,
+                         parallel_run=False,
+                         callback_whitelist=constants.ANSIBLE_CWL,
                          ansible_cfg=None, ansible_timeout=30,
                          reproduce_command=False, fail_on_rc=True,
                          timeout=None):
@@ -269,13 +271,14 @@ def run_ansible_playbook(playbook, inventory, workdir, playbook_dir=None,
     :param connection: Connection type (local, smart, etc).
     :type connection: String
 
-    :param output_callback: Callback for output format. Defaults to "json".
+    :param output_callback: Callback for output format. Defaults to
+                            "tripleo_dense".
     :type output_callback: String
 
     :param callback_whitelist: Comma separated list of callback plugins.
-                               Defaults to the value of `output_callback`.
-                               When the verbosity is > 0 "profile_tasks"
-                               will also be whitelisted.
+                               Defaults to
+                               "tripleo_dense,tripleo_profile_tasks".
+                               Custom output_callback is also whitelisted.
     :type callback_whitelist: String
 
     :param ssh_user: User for the ssh connection.
@@ -453,12 +456,8 @@ def run_ansible_playbook(playbook, inventory, workdir, playbook_dir=None,
     )
     makedirs(ansible_fact_path)
 
-    if callback_whitelist:
+    if output_callback not in callback_whitelist.split(','):
         callback_whitelist = ','.join([callback_whitelist, output_callback])
-    else:
-        callback_whitelist = output_callback
-
-    callback_whitelist = ','.join([callback_whitelist, 'profile_tasks'])
 
     env = dict()
     env['ANSIBLE_SSH_ARGS'] = (
@@ -477,7 +476,7 @@ def run_ansible_playbook(playbook, inventory, workdir, playbook_dir=None,
         '-T'
     ).format(os.devnull)
     env['ANSIBLE_DISPLAY_FAILED_STDERR'] = True
-    env['ANSIBLE_FORKS'] = 36
+    env['ANSIBLE_FORKS'] = multiprocessing.cpu_count() * 4
     env['ANSIBLE_TIMEOUT'] = ansible_timeout
     env['ANSIBLE_GATHER_TIMEOUT'] = 45
     env['ANSIBLE_SSH_RETRIES'] = 3
@@ -902,7 +901,7 @@ def create_tempest_deployer_input(config_name='tempest-deployer-input.conf'):
 
 
 def wait_for_stack_ready(orchestration_client, stack_name, marker=None,
-                         action='CREATE', verbose=False, nested_depth=2,
+                         action='CREATE', nested_depth=2,
                          max_retries=10):
     """Check the status of an orchestration stack
 
@@ -936,16 +935,13 @@ def wait_for_stack_ready(orchestration_client, stack_name, marker=None,
         return False
     stack_name = "%s/%s" % (stack.stack_name, stack.id)
 
-    if verbose:
-        out = sys.stdout
-    else:
-        out = open(os.devnull, "w")
     retries = 0
+
     while retries <= max_retries:
         try:
             stack_status, msg = event_utils.poll_for_events(
                 orchestration_client, stack_name, action=action,
-                poll_period=5, marker=marker, out=out,
+                poll_period=5, marker=marker, out=sys.stdout,
                 nested_depth=nested_depth)
             print(msg)
             return stack_status == '%s_COMPLETE' % action
@@ -959,9 +955,6 @@ def wait_for_stack_ready(orchestration_client, stack_name, marker=None,
                 continue
             log.error("Error occured while waiting for stack to be ready.")
             raise e
-        finally:
-            if not verbose:
-                out.close()
 
     raise RuntimeError(
         "wait_for_stack_ready: Max retries {} reached".format(max_retries))
@@ -1834,32 +1827,6 @@ def run_command_and_log(log, cmd, cwd=None, env=None, retcode_only=True):
         return proc
 
 
-def ffwd_upgrade_operator_confirm(parsed_args_yes, log):
-    print("\nWarning! The TripleO Fast Forward Upgrade "
-          "workflow is a critical operation against the deployed "
-          "environment.\nOnce and if you decide to use ffwd-upgrade "
-          "in production, ensure you are adequately prepared "
-          "with valid backup of your current deployment state.\n")
-    if parsed_args_yes:
-        log.debug(_("Fast forward upgrade --yes continuing"))
-        print(_("Continuing fast forward upgrade"))
-        return
-    else:
-        # Fix Python 2.x.
-        try:
-            input = raw_input
-        except NameError:
-            pass
-        response = input("Proceed with the fast forward upgrade? "
-                         "Type 'yes' to continue and anything else to "
-                         "cancel.\nConsider using the --yes parameter if "
-                         "you wish to skip this warning in future. ")
-        if response != 'yes':
-            log.debug(_("Fast forward upgrade cancelled on user request"))
-            print(_("Cancelling fast forward upgrade"))
-            sys.exit(1)
-
-
 def build_prepare_env(environment_files, environment_directories):
     '''Build the environment for container image prepare
 
