@@ -18,13 +18,15 @@ from oslo_log import log as logging
 from osc_lib.i18n import _
 from osc_lib import utils
 
+from tripleoclient.exceptions import OvercloudUpgradeNotConfirmed
+
 from tripleoclient import command
 from tripleoclient import constants
 from tripleoclient import exceptions
 from tripleoclient import utils as oooutils
 from tripleoclient.v1.overcloud_deploy import DeployOvercloud
 from tripleoclient.workflows import deployment
-from tripleoclient.workflows import package_update
+from tripleoclient.workflows import parameters
 
 CONF = cfg.CONF
 logging.register_options(CONF)
@@ -39,15 +41,32 @@ class UpgradePrepare(DeployOvercloud):
        first step for a major upgrade of your overcloud.
     """
 
-    log = logging.getLogger(__name__ + ".MajorUpgradePrepare")
+    operation = "Prepare"
+
+    template = constants.UPGRADE_PREPARE_ENV
+
+    forbidden_params = []
+
+    log = logging.getLogger(__name__ + ".UpgradePrepare")
 
     def get_parser(self, prog_name):
         parser = super(UpgradePrepare, self).get_parser(prog_name)
+        parser.add_argument('-y', '--yes', default=False,
+                            action='store_true',
+                            help=_("Use -y or --yes to skip the confirmation "
+                                   "required before any upgrade "
+                                   "operation. Use this with caution! "),
+                            )
         return parser
 
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)" % parsed_args)
 
+        if (not parsed_args.yes
+                and not oooutils.prompt_user_for_confirmation(
+                    constants.UPGRADE_PROMPT, self.log)):
+            raise OvercloudUpgradeNotConfirmed(constants.UPGRADE_NO)
+
         # Throw deprecation warning if service is enabled and
         # ask user if upgrade should still be continued.
         if parsed_args.environment_files:
@@ -62,25 +81,24 @@ class UpgradePrepare(DeployOvercloud):
         stack_name = stack.stack_name
 
         # In case of update and upgrade we need to force the
-        # update_plan_only. The heat stack update is done by the
-        # packag_update mistral action
-        parsed_args.update_plan_only = True
-        # Add the upgrade-prepare.yaml environment to set noops etc
+        # config_download to false. The heat stack update will be performed
+        # by DeployOvercloud class but skipping the config download part.
+        parsed_args.config_download = False
+        # Add the template attribute environment to set noops etc
         templates_dir = (parsed_args.templates or
                          constants.TRIPLEO_HEAT_TEMPLATES)
         parsed_args.environment_files = oooutils.prepend_environment(
             parsed_args.environment_files, templates_dir,
-            constants.UPGRADE_PREPARE_ENV)
+            self.template)
+        # Parse all environment files looking for undesired
+        # parameters
+        parameters.check_forbidden_params(self.log,
+                                          parsed_args.environment_files,
+                                          self.forbidden_params)
         super(UpgradePrepare, self).take_action(parsed_args)
-        package_update.update(clients, container=stack_name)
-        oooutils.get_config(
-            clients, container=stack_name,
-            container_config='{}-config'.format(stack.stack_name))
 
-        deployment.create_overcloudrc(container=stack_name)
-
-        # refresh stack info and enable ssh admin for Ansible-via-Mistral
-        stack = oooutils.get_stack(clients.orchestration, parsed_args.stack)
+        # enable ssh admin for Ansible-via-Mistral as that's done only
+        # when config_download is true
         deployment.get_hosts_and_enable_ssh_admin(
             stack,
             parsed_args.overcloud_ssh_network,
@@ -90,8 +108,8 @@ class UpgradePrepare(DeployOvercloud):
             verbosity=oooutils.playbook_verbosity(self=self)
         )
 
-        self.log.info("Completed Overcloud Upgrade Prepare for stack "
-                      "{0}".format(stack_name))
+        self.log.info("Completed Overcloud Upgrade {} for stack "
+                      "{}".format(self.operation, stack_name))
 
 
 class UpgradeRun(command.Command):
@@ -106,7 +124,7 @@ class UpgradeRun(command.Command):
        command is the second step in the major upgrade workflow.
     """
 
-    log = logging.getLogger(__name__ + ".MajorUpgradeRun")
+    log = logging.getLogger(__name__ + ".UpgradeRun")
 
     def get_parser(self, prog_name):
         parser = super(UpgradeRun, self).get_parser(prog_name)
@@ -173,13 +191,19 @@ class UpgradeRun(command.Command):
                             help=_('Name or ID of heat stack '
                                    '(default=Env: OVERCLOUD_STACK_NAME)'),
                             default=utils.env('OVERCLOUD_STACK_NAME',
-                                              default='overcloud'))
+                                              default='overcloud')
+                            )
         parser.add_argument('--no-workflow', dest='no_workflow',
                             action='store_true',
                             default=True,
                             help=_('This option no longer has any effect.')
                             )
-
+        parser.add_argument('-y', '--yes', default=False,
+                            action='store_true',
+                            help=_("Use -y or --yes to skip the confirmation "
+                                   "required before any upgrade "
+                                   "operation. Use this with caution! ")
+                            )
         return parser
 
     def _validate_skip_tags(self, skip_tags):
@@ -194,6 +218,12 @@ class UpgradeRun(command.Command):
 
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)" % parsed_args)
+
+        if (not parsed_args.yes
+                and not oooutils.prompt_user_for_confirmation(
+                    constants.UPGRADE_PROMPT, self.log)):
+            raise OvercloudUpgradeNotConfirmed(constants.UPGRADE_NO)
+
         # NOTE(cloudnull): The string option "all" was a special default
         #                  that is no longer relevant. To retain compatibility
         #                  this condition has been put in place.
@@ -232,7 +262,7 @@ class UpgradeRun(command.Command):
         self.log.info("Completed Overcloud Major Upgrade Run.")
 
 
-class UpgradeConvergeOvercloud(DeployOvercloud):
+class UpgradeConvergeOvercloud(UpgradePrepare):
     """Major upgrade converge - reset Heat resources in the stored plan
 
        This is the last step for completion of a overcloud major
@@ -241,25 +271,10 @@ class UpgradeConvergeOvercloud(DeployOvercloud):
        have set specific values for some stack Heat resources. This
        unsets those back to their default values.
     """
+    operation = "Converge"
+
+    template = constants.UPGRADE_CONVERGE_ENV
+
+    forbidden_params = constants.UPGRADE_CONVERGE_FORBIDDEN_PARAMS
 
     log = logging.getLogger(__name__ + ".UpgradeConvergeOvercloud")
-
-    def get_parser(self, prog_name):
-        parser = super(UpgradeConvergeOvercloud, self).get_parser(prog_name)
-        return parser
-
-    def take_action(self, parsed_args):
-        self.log.debug("take_action(%s)" % parsed_args)
-        clients = self.app.client_manager
-        stack = oooutils.get_stack(clients.orchestration,
-                                   parsed_args.stack)
-        # Add the converge environment into the args to unset noop etc
-        templates_dir = (parsed_args.templates or
-                         constants.TRIPLEO_HEAT_TEMPLATES)
-        parsed_args.environment_files = oooutils.prepend_environment(
-            parsed_args.environment_files, templates_dir,
-            constants.UPGRADE_CONVERGE_ENV)
-
-        super(UpgradeConvergeOvercloud, self).take_action(parsed_args)
-        self.log.info("Completed Overcloud Upgrade Converge for stack {0}"
-                      .format(stack.stack_name))
